local Me=CreateFrame("Frame","vZifnabsConfig")
vZifnabs.mod.config=Me
local mod=vZifnabs.mod

local mypfx="vZifnabs_Config"
local db=nil
local L={}
local myframe=nil

local button={set=0}

local active=nil
local unsaved_changes=false
local msg=nil

-- -----------------------------------------------------------------
-- INITIALIZE
-- -----------------------------------------------------------------
function Me:Initialize()
	db,L=mod.core:InitStart("config")
	myframe=getglobal(mypfx)
	mod.core:RegisterButton("config","show",L.button.config.show,"vZifnabs_Control_Config")
	mod.core:RegisterButton("config","hide",L.button.config.hide,mypfx.."_Hide")
	getglobal(mypfx.."_Heading"):SetText("vZifnabs Configuration")
	getglobal(mypfx.."_Release"):SetText(vZifnabs.release)	
	for b=1,5,1 do
		button[b]=getglobal(mypfx.."_"..b)
		button[b]:SetPoint("TOPLEFT",mypfx,"TOPLEFT",0,-20*b)
	end
	mod.core:RegisterButton("config","save",L.button.config.save,mypfx.."_Save",true)
	mod.core:RegisterButton("config","drop",L.button.config.drop,mypfx.."_Drop",true)
	mod.core:RegisterButton("config","continue",L.button.config.continue,mypfx.."_Continue",true)
	button.save=getglobal(mypfx.."_Save")
	button.drop=getglobal(mypfx.."_Drop")
	button.continue=getglobal(mypfx.."_Continue")
	mod.core:RegisterEvent("TICK","config",Me.Tick)
	msg=getglobal(mypfx.."_Unsaved")
	msg:SetText(L.msg.unsavedchanges)
	msg:Hide()
	mod.core:InitStop("config")
end

-- -----------------------------------------------------------------
-- TICK
-- -----------------------------------------------------------------
function Me:Tick()	
	if msg.flipflop then
		if msg.flipflop==-1 then 
			msg:Hide()
			msg.flipflop=nil
		else
			msg.flipflop=1-msg.flipflop+1
			if msg.flipflop==1 then msg:SetTextColor(1,0,0,1) else msg:SetTextColor(1,1,1,1) end
		end
	end
end

-- -----------------------------------------------------------------
-- REGISTER (a mod and panel to the config menu)
-- -----------------------------------------------------------------
function Me:Register(configmod,buttondef,panelname)
	button.set=button.set+1
	button[button.set].mod=configmod
	button[button.set].panel=getglobal(panelname)
	button[button.set].button_number=button.set
	mod.core:RegisterButton("config",button.set,buttondef,mypfx.."_"..button.set)
	button[button.set].panel:SetPoint("TOPRIGHT",mypfx,"TOPRIGHT",0,-20)
end

-- -----------------------------------------------------------------
-- SET UNSAVED CHANGES
-- -----------------------------------------------------------------
function Me:SetUnsavedChanges(status) 
	unsaved_changes=status 
	if unsaved_changes then
		button.save:Show()
		button.drop:Show()
	else
		button.save:Hide()
		button.drop:Hide()
		button.continue:Hide()
		msg.flipflop=-1
	end
end

-- -----------------------------------------------------------------
-- CLONE Table
-- -----------------------------------------------------------------
function Me:Clone(tbl)
	local ctbl={}
	for entry,data in tbl do
		if type(data)=="table" then 
			ctbl[entry]=Me:Clone(data)
		else
			ctbl[entry]=data
		end
	end
	return ctbl
end

-- -----------------------------------------------------------------
-- BUTTON pressed
-- -----------------------------------------------------------------
function Me:Button(pressedbutton,fnk)
	if fnk=="show" then
		myframe:Show()
		Me:Button(button[1],1)
	elseif fnk=="save" or fnk=="drop" then
		if fnk=="save" then
			active.mod:ConfigSave()
		else
			active.mod:ConfigDrop()
		end
		button.save:Hide()
		button.drop:Hide()
		button.continue:Hide()
		Me:SetUnsavedChanges(false)
		active.panel:Hide() -- to get it
		active.panel:Show() -- back to the top
	elseif fnk=="continue" then
		msg.flipflop=-1
		button.continue:Hide()
		active.panel:Hide() -- to get it
		active.panel:Show() -- back to the top
	elseif unsaved_changes then
		Me:WarnUnsavedChanges()
	elseif fnk=="hide" then 
		if active then Me:HideActive() end
		myframe:Hide()
	elseif button[fnk] then
		if active then Me:HideActive() end
		Me:ShowActive(pressedbutton)
	end
end

function Me:WarnUnsavedChanges()
	PlaySound("RaidWarning")
	msg:Show()
	msg.flipflop=0
	button.continue:Show()
end

function Me:HideActive()
	active.panel:Hide()
	active.mod:ConfigStop()
	active:Enable()
	active=nil
end
function Me:ShowActive(newactive)
	active=newactive
	active.mod:ConfigStart()
	active.panel:Show()
	active:Disable()
end

-- -----------------------------------------------------------------
-- INITIALIZE SLIDER
-- -----------------------------------------------------------------
function Me:InitializeSlider(slider,smin,smax,sstep,ddiv,title,tt)
	local holder={slider=getglobal(slider),text=getglobal(slider.."_Text"),high=getglobal(slider.."High"),low=getglobal(slider.."Low")}
	if holder.text==nil then holder.text=getglobal(slider.."Text") end
	holder.text:SetText(title)
	holder.low:SetText(smin/ddiv)
	holder.high:SetText(smax/ddiv)
	holder.slider:SetMinMaxValues(smin,smax)
	holder.slider:SetValueStep(sstep)
	holder.slider.tooltipText=tt
	holder.slider.valuedisplay=getglobal(slider.."_Value")
	holder.slider.displaydivide=ddiv
	if holder.slider.valuedisplay then 
		Me:Anchor(holder.slider.valuedisplay,"CENTER",slider,"CENTER",0,-14) 
		holder.slider.valuedisplay:SetTextColor(1,1,1,1)
		holder.slider.valuedisplay:SetText("?")
	end
	holder.downbutton=getglobal(slider.."_Down")
	if holder.downbutton then 
		Me:Anchor(holder.downbutton,"TOPRIGHT",slider,"TOPLEFT",0,0) 
		holder.downbutton.myslider=holder.slider
		holder.downbutton.direction=-1
	end
	holder.upbutton=getglobal(slider.."_Up")
	if holder.upbutton then 
		Me:Anchor(holder.upbutton,"TOPLEFT",slider,"TOPRIGHT",0,0) 
		holder.upbutton.myslider=holder.slider
		holder.upbutton.direction=1
	end
	return holder
end
function Me:Anchor(widget,wpoint,parent,ppoint,x,y)
	widget:ClearAllPoints()
	widget:SetPoint(wpoint,parent,ppoint,x,y)
end

-- -----------------------------------------------------------------
-- SLIDER CHANGE VALUE
-- -----------------------------------------------------------------
function Me:SliderValueChanged(widget)
	if widget.myslider then 
		local slider=widget.myslider
		local newvalue=slider:GetValue()+widget.direction*slider:GetValueStep()
		local smin,smax=slider:GetMinMaxValues()
		if newvalue>=smin and newvalue<=smax then 
			slider:SetValue(newvalue) 
		else 
			PlaySound("ITEMGENERICSOUND") 
		end
	else
		if widget.valuedisplay then widget.valuedisplay:SetText(widget:GetValue()/widget.displaydivide) end
		if widget.exitmodule then widget.exitmodule:SliderValueChanged(widget,widget:GetValue()/widget.displaydivide) end
	end
end
